@@ -1,26 +1,20 @@
 //===-- ABIMacOSX_arm64.cpp -------------------------------------*- C++ -*-===//
 //
-//                     The LLVM Compiler Infrastructure
-//
-// This file is distributed under the University of Illinois Open Source
-// License. See LICENSE.TXT for details.
+// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
+// See https://llvm.org/LICENSE.txt for license information.
+// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 //
 //===----------------------------------------------------------------------===//
 
 #include "ABIMacOSX_arm64.h"
 
-// C Includes
-// C++ Includes
 #include <vector>
 
-// Other libraries and framework includes
 #include "llvm/ADT/STLExtras.h"
 #include "llvm/ADT/Triple.h"
 
 #include "lldb/Core/Module.h"
 #include "lldb/Core/PluginManager.h"
-#include "lldb/Core/RegisterValue.h"
-#include "lldb/Core/Scalar.h"
 #include "lldb/Core/Value.h"
 #include "lldb/Core/ValueObjectConstResult.h"
 #include "lldb/Symbol/UnwindPlan.h"
@@ -30,6 +24,8 @@
 #include "lldb/Target/Thread.h"
 #include "lldb/Utility/ConstString.h"
 #include "lldb/Utility/Log.h"
+#include "lldb/Utility/RegisterValue.h"
+#include "lldb/Utility/Scalar.h"
 #include "lldb/Utility/Status.h"
 
 #include "Utility/ARM64_DWARF_Registers.h"
@@ -1661,21 +1657,16 @@ ABIMacOSX_arm64::GetRegisterInfoArray(uint32_t &count) {
 
 size_t ABIMacOSX_arm64::GetRedZoneSize() const { return 128; }
 
-//------------------------------------------------------------------
 // Static Functions
-//------------------------------------------------------------------
 
 ABISP
 ABIMacOSX_arm64::CreateInstance(ProcessSP process_sp, const ArchSpec &arch) {
-  static ABISP g_abi_sp;
   const llvm::Triple::ArchType arch_type = arch.GetTriple().getArch();
   const llvm::Triple::VendorType vendor_type = arch.GetTriple().getVendor();
 
   if (vendor_type == llvm::Triple::Apple) {
     if (arch_type == llvm::Triple::aarch64) {
-      if (!g_abi_sp)
-        g_abi_sp.reset(new ABIMacOSX_arm64(process_sp));
-      return g_abi_sp;
+      return ABISP(new ABIMacOSX_arm64(process_sp));
     }
   }
 
@@ -1768,90 +1759,92 @@ bool ABIMacOSX_arm64::GetArgumentValues(Thread &thread,
       return false;
 
     CompilerType value_type = value->GetCompilerType();
-    if (value_type) {
-      bool is_signed = false;
-      size_t bit_width = 0;
-      if (value_type.IsIntegerOrEnumerationType(is_signed)) {
-        bit_width = value_type.GetBitSize(&thread);
-      } else if (value_type.IsPointerOrReferenceType()) {
-        bit_width = value_type.GetBitSize(&thread);
-      } else {
-        // We only handle integer, pointer and reference types currently...
-        return false;
-      }
+    llvm::Optional<uint64_t> bit_size = value_type.GetBitSize(&thread);
+    if (!bit_size)
+      return false;
 
-      if (bit_width <= (exe_ctx.GetProcessRef().GetAddressByteSize() * 8)) {
-        if (value_idx < 8) {
-          // Arguments 1-6 are in x0-x5...
-          const RegisterInfo *reg_info = nullptr;
-          // Search by generic ID first, then fall back to by name
-          uint32_t arg_reg_num = reg_ctx->ConvertRegisterKindToRegisterNumber(
-              eRegisterKindGeneric, LLDB_REGNUM_GENERIC_ARG1 + value_idx);
-          if (arg_reg_num != LLDB_INVALID_REGNUM) {
-            reg_info = reg_ctx->GetRegisterInfoAtIndex(arg_reg_num);
-          } else {
-            switch (value_idx) {
-            case 0:
-              reg_info = reg_ctx->GetRegisterInfoByName("x0");
-              break;
-            case 1:
-              reg_info = reg_ctx->GetRegisterInfoByName("x1");
-              break;
-            case 2:
-              reg_info = reg_ctx->GetRegisterInfoByName("x2");
-              break;
-            case 3:
-              reg_info = reg_ctx->GetRegisterInfoByName("x3");
-              break;
-            case 4:
-              reg_info = reg_ctx->GetRegisterInfoByName("x4");
-              break;
-            case 5:
-              reg_info = reg_ctx->GetRegisterInfoByName("x5");
-              break;
-            case 6:
-              reg_info = reg_ctx->GetRegisterInfoByName("x6");
-              break;
-            case 7:
-              reg_info = reg_ctx->GetRegisterInfoByName("x7");
-              break;
-            }
-          }
+    bool is_signed = false;
+    size_t bit_width = 0;
+    if (value_type.IsIntegerOrEnumerationType(is_signed)) {
+      bit_width = *bit_size;
+    } else if (value_type.IsPointerOrReferenceType()) {
+      bit_width = *bit_size;
+    } else {
+      // We only handle integer, pointer and reference types currently...
+      return false;
+    }
 
-          if (reg_info) {
-            RegisterValue reg_value;
-
-            if (reg_ctx->ReadRegister(reg_info, reg_value)) {
-              if (is_signed)
-                reg_value.SignExtend(bit_width);
-              if (!reg_value.GetScalarValue(value->GetScalar()))
-                return false;
-              continue;
-            }
-          }
-          return false;
+    if (bit_width <= (exe_ctx.GetProcessRef().GetAddressByteSize() * 8)) {
+      if (value_idx < 8) {
+        // Arguments 1-6 are in x0-x5...
+        const RegisterInfo *reg_info = nullptr;
+        // Search by generic ID first, then fall back to by name
+        uint32_t arg_reg_num = reg_ctx->ConvertRegisterKindToRegisterNumber(
+            eRegisterKindGeneric, LLDB_REGNUM_GENERIC_ARG1 + value_idx);
+        if (arg_reg_num != LLDB_INVALID_REGNUM) {
+          reg_info = reg_ctx->GetRegisterInfoAtIndex(arg_reg_num);
         } else {
-          if (sp == 0) {
-            // Read the stack pointer if we already haven't read it
-            sp = reg_ctx->GetSP(0);
-            if (sp == 0)
+          switch (value_idx) {
+          case 0:
+            reg_info = reg_ctx->GetRegisterInfoByName("x0");
+            break;
+          case 1:
+            reg_info = reg_ctx->GetRegisterInfoByName("x1");
+            break;
+          case 2:
+            reg_info = reg_ctx->GetRegisterInfoByName("x2");
+            break;
+          case 3:
+            reg_info = reg_ctx->GetRegisterInfoByName("x3");
+            break;
+          case 4:
+            reg_info = reg_ctx->GetRegisterInfoByName("x4");
+            break;
+          case 5:
+            reg_info = reg_ctx->GetRegisterInfoByName("x5");
+            break;
+          case 6:
+            reg_info = reg_ctx->GetRegisterInfoByName("x6");
+            break;
+          case 7:
+            reg_info = reg_ctx->GetRegisterInfoByName("x7");
+            break;
+          }
+        }
+
+        if (reg_info) {
+          RegisterValue reg_value;
+
+          if (reg_ctx->ReadRegister(reg_info, reg_value)) {
+            if (is_signed)
+              reg_value.SignExtend(bit_width);
+            if (!reg_value.GetScalarValue(value->GetScalar()))
               return false;
+            continue;
           }
-
-          // Arguments 5 on up are on the stack
-          const uint32_t arg_byte_size = (bit_width + (8 - 1)) / 8;
-          Status error;
-          if (!exe_ctx.GetProcessRef().ReadScalarIntegerFromMemory(
-                  sp, arg_byte_size, is_signed, value->GetScalar(), error))
+        }
+        return false;
+      } else {
+        if (sp == 0) {
+          // Read the stack pointer if we already haven't read it
+          sp = reg_ctx->GetSP(0);
+          if (sp == 0)
             return false;
+        }
 
-          sp += arg_byte_size;
-          // Align up to the next 8 byte boundary if needed
-          if (sp % 8) {
-            sp >>= 3;
-            sp += 1;
-            sp <<= 3;
-          }
+        // Arguments 5 on up are on the stack
+        const uint32_t arg_byte_size = (bit_width + (8 - 1)) / 8;
+        Status error;
+        if (!exe_ctx.GetProcessRef().ReadScalarIntegerFromMemory(
+                sp, arg_byte_size, is_signed, value->GetScalar(), error))
+          return false;
+
+        sp += arg_byte_size;
+        // Align up to the next 8 byte boundary if needed
+        if (sp % 8) {
+          sp >>= 3;
+          sp += 1;
+          sp <<= 3;
         }
       }
     }
@@ -2115,13 +2108,12 @@ static bool LoadValueFromConsecutiveGPRRegisters(
     uint32_t &NGRN,       // NGRN (see ABI documentation)
     uint32_t &NSRN,       // NSRN (see ABI documentation)
     DataExtractor &data) {
-  const size_t byte_size = value_type.GetByteSize(nullptr);
-
-  if (byte_size == 0)
+  llvm::Optional<uint64_t> byte_size = value_type.GetByteSize(nullptr);
+  if (!byte_size || *byte_size == 0)
     return false;
 
-  std::unique_ptr<DataBufferHeap> heap_data_ap(
-      new DataBufferHeap(byte_size, 0));
+  std::unique_ptr<DataBufferHeap> heap_data_up(
+      new DataBufferHeap(*byte_size, 0));
   const ByteOrder byte_order = exe_ctx.GetProcessRef().GetByteOrder();
   Status error;
 
@@ -2133,7 +2125,9 @@ static bool LoadValueFromConsecutiveGPRRegisters(
     if (NSRN < 8 && (8 - NSRN) >= homogeneous_count) {
       if (!base_type)
         return false;
-      const size_t base_byte_size = base_type.GetByteSize(nullptr);
+      llvm::Optional<uint64_t> base_byte_size = base_type.GetByteSize(nullptr);
+      if (!base_byte_size)
+        return false;
       uint32_t data_offset = 0;
 
       for (uint32_t i = 0; i < homogeneous_count; ++i) {
@@ -2144,7 +2138,7 @@ static bool LoadValueFromConsecutiveGPRRegisters(
         if (reg_info == nullptr)
           return false;
 
-        if (base_byte_size > reg_info->byte_size)
+        if (*base_byte_size > reg_info->byte_size)
           return false;
 
         RegisterValue reg_value;
@@ -2152,12 +2146,12 @@ static bool LoadValueFromConsecutiveGPRRegisters(
         if (!reg_ctx->ReadRegister(reg_info, reg_value))
           return false;
 
-        // Make sure we have enough room in "heap_data_ap"
-        if ((data_offset + base_byte_size) <= heap_data_ap->GetByteSize()) {
+        // Make sure we have enough room in "heap_data_up"
+        if ((data_offset + *base_byte_size) <= heap_data_up->GetByteSize()) {
           const size_t bytes_copied = reg_value.GetAsMemoryData(
-              reg_info, heap_data_ap->GetBytes() + data_offset, base_byte_size,
+              reg_info, heap_data_up->GetBytes() + data_offset, *base_byte_size,
               byte_order, error);
-          if (bytes_copied != base_byte_size)
+          if (bytes_copied != *base_byte_size)
             return false;
           data_offset += bytes_copied;
           ++NSRN;
@@ -2166,16 +2160,16 @@ static bool LoadValueFromConsecutiveGPRRegisters(
       }
       data.SetByteOrder(byte_order);
       data.SetAddressByteSize(exe_ctx.GetProcessRef().GetAddressByteSize());
-      data.SetData(DataBufferSP(heap_data_ap.release()));
+      data.SetData(DataBufferSP(heap_data_up.release()));
       return true;
     }
   }
 
   const size_t max_reg_byte_size = 16;
-  if (byte_size <= max_reg_byte_size) {
-    size_t bytes_left = byte_size;
+  if (*byte_size <= max_reg_byte_size) {
+    size_t bytes_left = *byte_size;
     uint32_t data_offset = 0;
-    while (data_offset < byte_size) {
+    while (data_offset < *byte_size) {
       if (NGRN >= 8)
         return false;
 
@@ -2195,7 +2189,7 @@ static bool LoadValueFromConsecutiveGPRRegisters(
 
       const size_t curr_byte_size = std::min<size_t>(8, bytes_left);
       const size_t bytes_copied = reg_value.GetAsMemoryData(
-          reg_info, heap_data_ap->GetBytes() + data_offset, curr_byte_size,
+          reg_info, heap_data_up->GetBytes() + data_offset, curr_byte_size,
           byte_order, error);
       if (bytes_copied == 0)
         return false;
@@ -2239,15 +2233,15 @@ static bool LoadValueFromConsecutiveGPRRegisters(
       return false;
 
     if (exe_ctx.GetProcessRef().ReadMemory(
-            value_addr, heap_data_ap->GetBytes(), heap_data_ap->GetByteSize(),
-            error) != heap_data_ap->GetByteSize()) {
+            value_addr, heap_data_up->GetBytes(), heap_data_up->GetByteSize(),
+            error) != heap_data_up->GetByteSize()) {
       return false;
     }
   }
 
   data.SetByteOrder(byte_order);
   data.SetAddressByteSize(exe_ctx.GetProcessRef().GetAddressByteSize());
-  data.SetData(DataBufferSP(heap_data_ap.release()));
+  data.SetData(DataBufferSP(heap_data_up.release()));
   return true;
 }
 
@@ -2267,7 +2261,10 @@ ValueObjectSP ABIMacOSX_arm64::GetReturnValueObjectImpl(
   if (!reg_ctx)
     return return_valobj_sp;
 
-  const size_t byte_size = return_compiler_type.GetByteSize(nullptr);
+  llvm::Optional<uint64_t> byte_size =
+      return_compiler_type.GetByteSize(nullptr);
+  if (!byte_size)
+    return return_valobj_sp;
 
   const uint32_t type_flags = return_compiler_type.GetTypeInfo(nullptr);
   if (type_flags & eTypeIsScalar || type_flags & eTypeIsPointer) {
@@ -2276,7 +2273,7 @@ ValueObjectSP ABIMacOSX_arm64::GetReturnValueObjectImpl(
     bool success = false;
     if (type_flags & eTypeIsInteger || type_flags & eTypeIsPointer) {
       // Extract the register context so we can read arguments from registers
-      if (byte_size <= 8) {
+      if (*byte_size <= 8) {
         const RegisterInfo *x0_reg_info =
             reg_ctx->GetRegisterInfoByName("x0", 0);
         if (x0_reg_info) {
@@ -2284,7 +2281,7 @@ ValueObjectSP ABIMacOSX_arm64::GetReturnValueObjectImpl(
               thread.GetRegisterContext()->ReadRegisterAsUnsigned(x0_reg_info,
                                                                   0);
           const bool is_signed = (type_flags & eTypeIsSigned) != 0;
-          switch (byte_size) {
+          switch (*byte_size) {
           default:
             break;
           case 16: // uint128_t
@@ -2294,10 +2291,10 @@ ValueObjectSP ABIMacOSX_arm64::GetReturnValueObjectImpl(
                   reg_ctx->GetRegisterInfoByName("x1", 0);
 
               if (x1_reg_info) {
-                if (byte_size <=
+                if (*byte_size <=
                     x0_reg_info->byte_size + x1_reg_info->byte_size) {
-                  std::unique_ptr<DataBufferHeap> heap_data_ap(
-                      new DataBufferHeap(byte_size, 0));
+                  std::unique_ptr<DataBufferHeap> heap_data_up(
+                      new DataBufferHeap(*byte_size, 0));
                   const ByteOrder byte_order =
                       exe_ctx.GetProcessRef().GetByteOrder();
                   RegisterValue x0_reg_value;
@@ -2306,13 +2303,13 @@ ValueObjectSP ABIMacOSX_arm64::GetReturnValueObjectImpl(
                       reg_ctx->ReadRegister(x1_reg_info, x1_reg_value)) {
                     Status error;
                     if (x0_reg_value.GetAsMemoryData(
-                            x0_reg_info, heap_data_ap->GetBytes() + 0, 8,
+                            x0_reg_info, heap_data_up->GetBytes() + 0, 8,
                             byte_order, error) &&
                         x1_reg_value.GetAsMemoryData(
-                            x1_reg_info, heap_data_ap->GetBytes() + 8, 8,
+                            x1_reg_info, heap_data_up->GetBytes() + 8, 8,
                             byte_order, error)) {
                       DataExtractor data(
-                          DataBufferSP(heap_data_ap.release()), byte_order,
+                          DataBufferSP(heap_data_up.release()), byte_order,
                           exe_ctx.GetProcessRef().GetAddressByteSize());
 
                       return_valobj_sp = ValueObjectConstResult::Create(
@@ -2362,7 +2359,7 @@ ValueObjectSP ABIMacOSX_arm64::GetReturnValueObjectImpl(
       if (type_flags & eTypeIsComplex) {
         // Don't handle complex yet.
       } else {
-        if (byte_size <= sizeof(long double)) {
+        if (*byte_size <= sizeof(long double)) {
           const RegisterInfo *v0_reg_info =
               reg_ctx->GetRegisterInfoByName("v0", 0);
           RegisterValue v0_value;
@@ -2370,13 +2367,13 @@ ValueObjectSP ABIMacOSX_arm64::GetReturnValueObjectImpl(
             DataExtractor data;
             if (v0_value.GetData(data)) {
               lldb::offset_t offset = 0;
-              if (byte_size == sizeof(float)) {
+              if (*byte_size == sizeof(float)) {
                 value.GetScalar() = data.GetFloat(&offset);
                 success = true;
-              } else if (byte_size == sizeof(double)) {
+              } else if (*byte_size == sizeof(double)) {
                 value.GetScalar() = data.GetDouble(&offset);
                 success = true;
-              } else if (byte_size == sizeof(long double)) {
+              } else if (*byte_size == sizeof(long double)) {
                 value.GetScalar() = data.GetLongDouble(&offset);
                 success = true;
               }
@@ -2390,22 +2387,22 @@ ValueObjectSP ABIMacOSX_arm64::GetReturnValueObjectImpl(
       return_valobj_sp = ValueObjectConstResult::Create(
           thread.GetStackFrameAtIndex(0).get(), value, ConstString(""));
   } else if (type_flags & eTypeIsVector) {
-    if (byte_size > 0) {
+    if (*byte_size > 0) {
 
       const RegisterInfo *v0_info = reg_ctx->GetRegisterInfoByName("v0", 0);
 
       if (v0_info) {
-        if (byte_size <= v0_info->byte_size) {
-          std::unique_ptr<DataBufferHeap> heap_data_ap(
-              new DataBufferHeap(byte_size, 0));
+        if (*byte_size <= v0_info->byte_size) {
+          std::unique_ptr<DataBufferHeap> heap_data_up(
+              new DataBufferHeap(*byte_size, 0));
           const ByteOrder byte_order = exe_ctx.GetProcessRef().GetByteOrder();
           RegisterValue reg_value;
           if (reg_ctx->ReadRegister(v0_info, reg_value)) {
             Status error;
-            if (reg_value.GetAsMemoryData(v0_info, heap_data_ap->GetBytes(),
-                                          heap_data_ap->GetByteSize(),
+            if (reg_value.GetAsMemoryData(v0_info, heap_data_up->GetBytes(),
+                                          heap_data_up->GetByteSize(),
                                           byte_order, error)) {
-              DataExtractor data(DataBufferSP(heap_data_ap.release()),
+              DataExtractor data(DataBufferSP(heap_data_up.release()),
                                  byte_order,
                                  exe_ctx.GetProcessRef().GetAddressByteSize());
               return_valobj_sp = ValueObjectConstResult::Create(
@@ -2440,9 +2437,7 @@ void ABIMacOSX_arm64::Terminate() {
   PluginManager::UnregisterPlugin(CreateInstance);
 }
 
-//------------------------------------------------------------------
 // PluginInterface protocol
-//------------------------------------------------------------------
 
 ConstString ABIMacOSX_arm64::GetPluginNameStatic() {
   static ConstString g_plugin_name("ABIMacOSX_arm64");
